/**
 * 
 */
package de.ixxat.vci3.bal.can;

import de.ixxat.ZeroHex;

/**
 * This value class represents a bit rate for CAN busses. A CAN bit rate
 * is defined of two bit timing register values: Btr0 and Btr1. The standardized
 * CiA bit timing register values are provided as static CanBitrate members like
 * Cia250KBit.
 * 
 * 
 * @author Michael Ummenhofer
 * 
 */
public class CanBitrate
{
  /**
   * bit timing value for the predefined CiA bit rate of 10 kbit/s.
   */ 
  public static final int Cia10KBit   = 10000;
  /**
   * bit timing value for the predefined CiA bit rate of 20 kbit/s.
   */ 
  public static final int Cia20KBit   = 20000;
  /**
   * bit timing value for the predefined CiA bit rate of 50 kbit/s.
   */ 
  public static final int Cia50KBit   = 50000;
  /**
   * bit timing value for the predefined CiA bit rate of 125 kbit/s.
   */ 
  public static final int Cia125KBit  = 125000;
  /**
   * bit timing value for the predefined CiA bit rate of 250 kbit/s.
   */ 
  public static final int Cia250KBit  = 250000;
  /**
   * bit timing value for the predefined CiA bit rate of 500 kbit/s.
   */ 
  public static final int Cia500KBit  = 500000;
  /**
   * bit timing value for the predefined CiA bit rate of 800 kbit/s.
   */ 
  public static final int Cia800KBit  = 800000;
  /**
   * bit timing value for the predefined CiA bit rate of 1000 kbit/s.
   */ 
  public static final int Cia1000KBit = 1000000;
  
  /**
   * Bus timing register 0
   */
  private byte m_bBtr0 = 0;

  /**
   * Bus timing register 1
   */
  private byte m_bBtr1 = 0;

  /**
   * Creates a new CanBitrate using the given BaudRate
   */
  public CanBitrate(int dwBaudrate) throws Throwable
  {
    // Switch on BitRate
    switch(dwBaudrate)
    {
      case Cia10KBit:
        {m_bBtr0 = 0x31; m_bBtr1 = 0x1c; break;}
      case Cia20KBit:
        {m_bBtr0 = 0x18; m_bBtr1 = 0x1c; break;}
      case Cia50KBit:
        {m_bBtr0 = 0x09; m_bBtr1 = 0x1c; break;}
      case Cia125KBit:
        {m_bBtr0 = 0x03; m_bBtr1 = 0x1c; break;}
      case Cia250KBit:
        {m_bBtr0 = 0x01; m_bBtr1 = 0x1c; break;}
      case Cia500KBit:
        {m_bBtr0 = 0x00; m_bBtr1 = 0x1c; break;}
      case Cia800KBit:
        {m_bBtr0 = 0x00; m_bBtr1 = 0x16; break;}
      case Cia1000KBit:
        {m_bBtr0 = 0x00; m_bBtr1 = 0x14; break;}
      default:
      {
        throw new IllegalArgumentException("dwBaudrate out of range");
      }
    }
  }

  /**
   * Creates a new CanBitrate using the given bus timing registers
   */
  public CanBitrate(byte bBtr0, byte bBtr1)
  {
    m_bBtr0 = bBtr0;
    m_bBtr1 = bBtr1;
  }
  
  /**
   * Gets the value of the bus timing register 0. The value corresponds to the
   * BTR0 register of the Philips SJA 1000 CAN controller with a cycle frequency
   * of 16 MHz. Further information on this is given in the data sheet of the SJA
   * 1000.
   * 
   * @return Bus timing register 0
   */
  public byte get_bBtr0()
  {
    return m_bBtr0;
  }

  /**
   * Gets the value of the bus timing register 1. The value corresponds to the
   * BTR1 register of the Philips SJA 1000 CAN controller with a cycle frequency
   * of 16 MHz. Further information on this is given in the data sheet of the SJA
   * 1000.
   * 
   * @return Bus timing register 1
   */
  public byte get_bBtr1()
  {
    return m_bBtr1;
  }
  
  /**
   * Returns the CAN BAUD Rate as string.
   * @return The CAN BAUD Rate as string.
   */
  public String toString()
  {
    String strCanLineStatus = "";
    
    strCanLineStatus += "BtReg0: 0x" + ZeroHex.toZeroHexString(m_bBtr0) + ", ";
    strCanLineStatus += "BtReg1: 0x" + ZeroHex.toZeroHexString(m_bBtr1) + ", ";
    
    return strCanLineStatus;
  }

}
