/**
 * 
 */
package de.ixxat.vci3.bal.can;

import de.ixxat.ZeroHex;
import de.ixxat.jni.CPointer;

/**
 * This value class represents a CAN message object. CAN messages can be received and
 * transmitted via the message reader {@link ICanMessageReader} and the message writer
 * {@link ICanMessageWriter} of a CAN channel {@link ICanChannel}. 
 * 
 * @author Michael Ummenhofer
 * 
 */

public class CanMessage
{
  /** data frame */
  protected static final short CAN_MSGTYPE_DATA = 0;
  /** info frame */
  protected static final short CAN_MSGTYPE_INFO = 1;
  /** error frame */
  protected static final short CAN_MSGTYPE_ERROR = 2;
  /** status frame */
  protected static final short CAN_MSGTYPE_STATUS = 3;
  /** wakeup frame */
  protected static final short CAN_MSGTYPE_WAKEUP = 4;
  /** timer overrun */
  protected static final short CAN_MSGTYPE_TIMEOVR = 5;
  /** timer reset */
  protected static final short CAN_MSGTYPE_TIMERST = 6;

  /** data length code */
  protected static final short CAN_MSGFLAGS_DLC = 0x0F;
  /** data overrun flag */
  protected static final short CAN_MSGFLAGS_OVR = 0x10;
  /** self reception request */
  protected static final short CAN_MSGFLAGS_SRR = 0x20;
  /** remote transmission request */
  protected static final short CAN_MSGFLAGS_RTR = 0x40;
  /** frame format (0=11-bit, 1=29-bit) */
  protected static final short CAN_MSGFLAGS_EXT = 0x80;

  /**
   * With receive messages, this field contains the relative reception time of
   * the message in ticks. The resolution of a tick can be calculated from the
   * properties {@link CanCapabilities#m_dwClockFreq} and
   * {@link CanCapabilities#m_dwTscDivisor}. in accordance with the following
   * formula:
   * <p>
   * <blockquote><code> 
   * Resolution [s] = TimeStampCounterDivisor / ClockFrequency
   * </code></blockquote>
   * <p>
   * With transmit messages, the field defines with how many ticks delay the
   * message is to be transmitted to the bus. The delay time between the last
   * message transmitted and the new message can be calculated with the
   * properties {@link CanCapabilities#m_dwClockFreq} and
   * {@link CanCapabilities#m_dwDtxDivisor} in accordance with the following
   * formula:
   * <p>
   * <blockquote><code> 
   * delay time [s] = (DelayedTXTimerDivisor / ClockFrequency) * TimeStamp 
   * </code></blockquote>
   * <p>
   * The maximum possible delay time is defined by the property
   * {@link CanCapabilities#m_dwDtxMaxTicks}.
   */
  public int m_dwTimestamp = 0;

  /**
   * Gets or sets the CAN identifier of this CAN message.
   */
  public int m_dwIdentifier = 0;

  /**
   * Gets or sets the data length of this CAN message.
   */
  public byte m_bDataLength = 0;

  /**
   * Gets or sets a single data byte at the specified index from this CAN
   * message. 
   */
  public byte[] m_abData = new byte[8];

  /**
   * Gets or sets a value indicating whether this message has extended frame
   * format (29-bit id).
   * <p>
   * For {@link CanMessage#toString()}: <code>"E"</code> means extended frame format 
   */
  public boolean m_fExtendedFrameFormat = false;

  /**
   * Gets or sets a value indicating whether this message is the result of a
   * self reception request.
   * <p>
   * For {@link CanMessage#toString()}: <code>"S"</code> means self reception message 
   */
  public boolean m_fSelfReception = false;

  /**
   * Gets a value indicating whether this message was the last message which
   * could be placed in the receive FIFO, before this overflows.
   * <p>
   * For {@link CanMessage#toString()}: <code>"O"</code> means possible overrun 
   */
  public boolean m_fPossibleOverrun = false;

  /**
   * Gets or sets a value indicating whether this message is a remote
   * transmission request.
   * <p>
   * For {@link CanMessage#toString()}: <code>"R"</code> means remote transmission request 
   */
  public boolean m_fRemoteTransmissionRequest = false;

  /**
   * Constructs a new empty CAN Message Object.
   */
  public CanMessage()
  {
  }

  /**
   * Constructs a new CAN Message Object using the CAN Object given as CPointer.
   * 
   * @param pMsg
   *          CPointer Object which holds a C-Code Memory location with a CANMSG
   *          structure.
   * @throws Throwable
   */
  protected CanMessage(CPointer pMsg) throws Throwable
  {
    fromCPointer(pMsg);
  }

  protected void finalize() throws Throwable
  {
    try
    {
    } finally
    {
      // Call finalize of base class
      super.finalize();
    }
  }

  /**
   * Update a existing CAN Message Object using the CAN Object given as
   * CPointer.
   * 
   * @param pMsg
   *          CPointer Object which holds a C-Code Memory location with a CANMSG
   *          structure.
   * @throws Throwable
   */
  protected void fromCPointer(CPointer pMsg) throws Throwable
  {
    byte bType      = 0;
    byte bReserved  = 0;
    byte bFlags     = 0;
    byte bAccept    = 0;

    // Get CAN Message Info
    bType     = pMsg.getByte(8);
    bReserved = pMsg.getByte(9);
    bFlags    = pMsg.getByte(10);
    bAccept   = pMsg.getByte(11);
    
    bReserved = 0;
    bAccept = 0;

    // TODO: Momentarily data frames supported, only!
    // Is data frame?
    // all other messages will throw an exception which leads to a removal of
    // the message
    if (bType != (byte) CAN_MSGTYPE_DATA)
      throw new IllegalArgumentException("Message is not CAN_MSGTYPE_DATA");

    // Get TimeStamp
    m_dwTimestamp = pMsg.getInt(0);

    // Get Can Message Identifier
    m_dwIdentifier = pMsg.getInt(4);

    // data length code
    m_bDataLength = (byte) (bFlags & (byte) CAN_MSGFLAGS_DLC);

    // data overrun flag
    if ((byte) (bFlags & (byte) CAN_MSGFLAGS_OVR) != 0)
      m_fPossibleOverrun = true;
    else
      m_fPossibleOverrun = false;

    // self reception
    if ((byte) (bFlags & (byte) CAN_MSGFLAGS_SRR) != 0)
      m_fSelfReception = true;
    else
      m_fSelfReception = false;

    // remote transmission request
    if ((byte) (bFlags & (byte) CAN_MSGFLAGS_RTR) != 0)
      m_fRemoteTransmissionRequest = true;
    else
      m_fRemoteTransmissionRequest = false;

    // frame format (0=11-bit, 1=29-bit)
    if ((byte) (bFlags & (byte) CAN_MSGFLAGS_EXT) != 0)
      m_fExtendedFrameFormat = true;
    else
      m_fExtendedFrameFormat = false;

    // Get CAN Message Data
    pMsg.copyOut(12, m_abData, 0, m_bDataLength);
  }

  /**
   * Updates the given CPointer memory structure with the current CanMessage
   * contents.
   * 
   * @param pMsg
   *          CPointer Object which holds a C-Code Memory location with a CANMSG
   *          structure.
   * @throws Throwable
   */
  protected void toCPointer(CPointer pMsg) throws Throwable
  {
    byte bType = 0; // CAN_MSGTYPE_DATA
    byte bReserved = 0; // Always 0
    byte bFlags = 0;
    byte bAccept = 0; // Always 0

    // TimeStamp now serves as message delay
    // set to zero to send the message immediately
    pMsg.setInt(0, m_dwTimestamp);

    // Set Can Message Identifier
    pMsg.setInt(4, m_dwIdentifier);

    // data frame
    bType = (byte) CAN_MSGTYPE_DATA;

    // data length code
    bFlags |= (m_bDataLength & (byte) CAN_MSGFLAGS_DLC);

    // self reception
    if (m_fSelfReception)
      bFlags |= (byte) CAN_MSGFLAGS_SRR;

    // remote transmission request
    if (m_fRemoteTransmissionRequest)
      bFlags |= (byte) CAN_MSGFLAGS_RTR;

    // Extended Frame Format
    if (m_fExtendedFrameFormat)
      bFlags |= (byte) CAN_MSGFLAGS_EXT;

    // Set CAN Message Info
    pMsg.setByte(8, bType);
    pMsg.setByte(9, bReserved);
    pMsg.setByte(10, bFlags);
    pMsg.setByte(11, bAccept);

    // Set CAN Message Data
    pMsg.copyIn(12, m_abData, 0, m_bDataLength);
  }

  /**
   * Returns a string of the current message.
   * @return String
   */
  public String toString()
  {
    String strMsg = "";
    String strFlags = "";

    // Possible overrun?
    if (m_fPossibleOverrun)
      strFlags += "O";
    else
      strFlags += " ";

    // RTR Message?
    if (m_fRemoteTransmissionRequest)
      strFlags += "R";
    else
      strFlags += " ";

    // Self Reception Message?
    if (m_fSelfReception)
      strFlags += "S";
    else
      strFlags += " ";

    // Extended Frame Format?
    if (m_fExtendedFrameFormat)
      strFlags += "E";
    else
      strFlags += " ";

    // Build Message
    strMsg += "Timestamp: " + m_dwTimestamp + " Flags: " + strFlags + " ID: 0x"
        + ZeroHex.toZeroHexString(m_dwIdentifier);

    // Data available?
    if ((m_bDataLength > 0) && !m_fRemoteTransmissionRequest)
    {
      strMsg += " Data:";
      for (byte bIndex = 0; bIndex < m_bDataLength; bIndex++)
      {
        strMsg += " 0x" + ZeroHex.toZeroHexString(m_abData[bIndex]);
      } // end for
    } // end if
    else if (m_fRemoteTransmissionRequest)
    {
      strMsg += " DLC = " + m_bDataLength;
    }

    return strMsg;
  }
}
