/**
 * 
 */
package de.ixxat.vci3.bal.lin;

import de.ixxat.vci3.IDisposable;

/**
 * This interface represents a LIN monitor channel and is used to receive LIN
 * messages. When no longer needed the LIN monitor channel object has to be
 * disposed using the {@link IDisposable} interface. A LIN monitor channel can
 * be obtained via method {@link ILinSocket#CreateMonitor(boolean)}. Unless the
 * LIN monitor channel is not already in use exclusively, it's possible to open
 * several concurrently monitor channels on the same socket.
 * 
 * @author Michael Ummenhofer
 */
public interface ILinMonitor extends IDisposable
{
  /**
   * This method initializes the LIN channel. This method must be called prior
   * to any other method of the interface.
   * <p>
   * <strong>Remarks:</strong> <blockquote> The channel is deactivated after
   * this method returns an must be activated by an {@link #Activate()} method
   * call. The method can be called more than once to reconfigure the size of
   * the receive FIFOs.</blockquote>
   * 
   * @param wRxSize
   *          Size of the receive buffer (number of CAN messages)
   * @throws Throwable
   */
  public void Initialize(short wRxSize) throws Throwable;

  /**
   * This method activates the LIN monitor. After activating the monitor, LIN
   * messages can be received through the message reader.
   * <p>
   * <strong>Remarks:</strong> <blockquote> The LIN line must be started,
   * otherwise no messages are received from the LIN line (see also
   * {@link ILinControl#StartLine()}).</blockquote>
   * 
   * @throws Throwable
   */
  public void Activate() throws Throwable;

  /**
   * This method deactivates the LIN monitor. After deactivating the monitor, no
   * further LIN messages are received from the LIN line.
   * 
   * @throws Throwable
   */
  public void Deactivate() throws Throwable;

  /**
   * Gets a reference to a new instance of a message reader object for the
   * monitor. This message reader provides access to the monitors's receive
   * buffer. LIN messages received from the LIN line can be read from this
   * object.
   * 
   * @return A reference to the message reader of the monitor.
   * @throws Throwable
   */
  public ILinMessageReader GetMessageReader() throws Throwable;

  /**
   * Gets the current status of the LIN channel.
   * 
   * @return {@link LinMonitorStatus}
   * @throws Throwable
   */
  public LinMonitorStatus GetStatus() throws Throwable;
}
